EpwMorphBackend defines a statistical downscaling backend that can be selected by epw_morph_recipe() and executed by EpwMorpher.

Public fields

name

Backend name.

label

Human-readable backend label.

requires_reference

Whether external reference climate data are mandatory for this backend.

accepts_reference

Whether the backend can consume external reference climate data. A backend may accept a reference without requiring one.

Methods


Method new()

Create an EPW morphing backend.

Usage

EpwMorphBackend$new(
  name,
  label = NULL,
  methods = NULL,
  method_choices = NULL,
  rules,
  requires_reference = FALSE,
  accepts_reference = requires_reference,
  pipeline = NULL,
  runner = NULL
)

Arguments

name

Backend name.

label

Human-readable backend label.

methods

Named default method vector.

method_choices

Allowed method values.

rules

Backend rule table.

requires_reference

Whether external reference climate data are mandatory. This can be TRUE only when accepts_reference is also TRUE.

accepts_reference

Whether external reference climate data can be consumed. TRUE with requires_reference = FALSE defines an optional-reference backend such as Belcher.

pipeline

Optional internal component pipeline specification.

runner

Optional function taking (context, backend) and returning an epw_morph_result. Exactly one of pipeline and runner must be supplied.


Method methods()

Return default backend methods.

Usage

EpwMorphBackend$methods()


Method method_choices()

Return allowed backend method values.

Usage

EpwMorphBackend$method_choices()


Method rules()

Return backend rules.

Usage

EpwMorphBackend$rules()


Method component_pipeline()

Return the optional component pipeline used by this backend.

Usage

EpwMorphBackend$component_pipeline()


Method required_variables()

Return required CMIP variable IDs.

Usage

EpwMorphBackend$required_variables()


Method validate_methods()

Validate and complete method overrides.

Usage

EpwMorphBackend$validate_methods(methods = NULL)

Arguments

methods

Optional named method override vector.


Method rules_with_methods()

Return backend rules with methods applied.

Usage

EpwMorphBackend$rules_with_methods(methods = NULL)

Arguments

methods

Optional named method override vector.


Method run()

Run this backend on a canonical EPW morphing context.

Usage

EpwMorphBackend$run(context)

Arguments

context

Canonical EPW morphing context.


Method clone()

The objects of this class are cloneable with this method.

Usage

EpwMorphBackend$clone(deep = FALSE)

Arguments

deep

Whether to make a deep clone.