esg_result() creates an empty query result object of input type, so that you can load the saved JSON file via EsgResult$load().

esg_result(type = c("dataset", "file", "aggregation"))

Arguments

type

A string indicating what type of ESGF query result should be created. Should be one of "dataset", "file" or "aggregation"`.

Value

An empty EsgResult object of given type.