store_dir() returns the root directory used for persistent epwshiftr store artifacts, including query snapshots, dictionaries, sources, downloads, extracted data, generated outputs, and the DuckDB manifest.

store_dir(init = TRUE)

Arguments

init

If TRUE, create the directory when it does not exist.

Value

A single string indicating the directory location.