weather_transforms() lists the same public scale, method, reconstruction, source-frequency, input-role, evidence, and output contracts used by the three transform constructors.

weather_transforms()

Value

A data table with one row per selectable configuration.