EpwMorphBackend defines a statistical downscaling backend that can be selected by epw_morph_recipe() and executed by EpwMorpher.

Public fields

name

Backend name.

label

Human-readable backend label.

requires_reference

Whether the backend requires reference climate data.

Methods


Method new()

Create an EPW morphing backend.

Usage

EpwMorphBackend$new(
  name,
  label = NULL,
  methods = NULL,
  method_choices = NULL,
  rules,
  requires_reference = FALSE,
  runner
)

Arguments

name

Backend name.

label

Human-readable backend label.

methods

Named default method vector.

method_choices

Allowed method values.

rules

Backend rule table.

requires_reference

Whether reference climate data are required.

runner

Function taking (context, backend) and returning an epw_morph_result.


Method methods()

Return default backend methods.

Usage

EpwMorphBackend$methods()


Method method_choices()

Return allowed backend method values.

Usage

EpwMorphBackend$method_choices()


Method rules()

Return backend rules.

Usage

EpwMorphBackend$rules()


Method required_variables()

Return required CMIP variable IDs.

Usage

EpwMorphBackend$required_variables()


Method validate_methods()

Validate and complete method overrides.

Usage

EpwMorphBackend$validate_methods(methods = NULL)

Arguments

methods

Optional named method override vector.


Method rules_with_methods()

Return backend rules with methods applied.

Usage

EpwMorphBackend$rules_with_methods(methods = NULL)

Arguments

methods

Optional named method override vector.


Method run()

Run this backend on a canonical EPW morphing context.

Usage

EpwMorphBackend$run(context)

Arguments

context

Canonical EPW morphing context.


Method clone()

The objects of this class are cloneable with this method.

Usage

EpwMorphBackend$clone(deep = FALSE)

Arguments

deep

Whether to make a deep clone.