R/epwshiftr-package.R
epwshiftr-package.RdQuery, download climate change projection data from the CMIP6 (Coupled Model Intercomparison Project Phase 6) project in the ESGF (Earth System Grid Federation) platform, and create future EnergyPlus Weather (EPW) files adjusted from climate changes using data from Global Climate Models (GCM).
epwshiftr.verbose: If TRUE, more detailed message will be printed.
Default: FALSE.
epwshiftr.progress: If TRUE, progress bars are shown for long-running
operations that support them. Default: interactive().
epwshiftr.threshold_alpha: the threshold of the absolute value for alpha,
i.e. monthly-mean fractional change, when performing morphing operations.
The default value is set to 3. If the morphing methods are set
"stretch" or "combined", and the absolute alpha exceeds the threshold
value, warnings are issued and the morphing method fallbacks to
"shift" to avoid unrealistic morphed values.
epwshiftr.dir_store: The persistent store directory for query snapshots,
dictionaries, source mirrors, downloads, extraction results, outputs, and
the store manifest. If not set, tools::R_user_dir() with type "data"
will be used.
epwshiftr.cache: Controls caching behavior. TRUE enables normal
caching (default), FALSE disables caching entirely, and "offline"
enables offline mode where only cached data is used and no network
requests are made. Default: TRUE
epwshiftr.dir_cache: The directory for disposable cache entries. Deleting
this directory can require re-fetching or re-parsing data, but should not
invalidate a persistent store.
Useful links: